<?php
namespace FLXDashBoardGroups;

class PagesGroupsModule{
    
    public $groupsTable;
    public $addForm;
    public $assignationsForm;
    
    public $groups;

    public static $exampleGroups = [
        '📄 Templates',
        '🔒 Private',
        '📁 Documents',
    ];
    
    function __construct(){
        $this->groups = SettingsModule::$data['PAGE_GROUPS_OPTION'];
        if(!empty($_POST['flxpagegroups_action'])) $this->onAction();
        
        $this->setGroupsTable($this->groups);
        $this->setAddForm();
        add_action('views_edit-page', [$this, 'pageGroupsFilter']);
        add_action('pre_get_posts', [$this, 'pageFilter']);
        
        add_filter( 'bulk_actions-edit-page', function($actions){
            return $this->addPagesBulkActions($actions, $this->groups);
        });
        add_filter('manage_pages_columns', [$this, 'addPageGroupColumns'], 10, 2);
        add_filter('manage_pages_custom_column', [$this, 'addPageGroupCell'], 10, 2);
        add_filter('handle_bulk_actions-edit-page', [$this, 'onAssignGroup'], 10, 3);
    }
    
    private function getGroupsCounts($status = false){
        global $wpdb;
        $status = (!$status || ($status == 'all')) ? '' : "AND post.post_status = '".$status."'";
        $count = $wpdb->get_results(
            "SELECT pm.meta_value as group_name, COUNT(DISTINCT pm.post_id) as count
            FROM {$wpdb->posts} post
            INNER JOIN {$wpdb->postmeta} pm
            ON pm.post_id = post.ID
            AND pm.meta_key = '".SettingsModule::PAGE_GROUP_META."'
            WHERE post.post_type = 'page'
            ".$status."
            GROUP BY pm.meta_value",
            OBJECT_K
        ); 
        return $count;
    }

    private function onAction(){
        $action = array_keys($_POST['flxpagegroups_action'])[0];
        switch($action){
            default: break;
            
            case 'remove_page_group':
                $this->groups = $this->onRemoveGroup($this->groups);
                break;
            
            case 'add_page_group':
                $this->groups = $this->onCreateGroup($this->groups);
                break;
            
            case 'flxdg-insert-default-page-groups':
                foreach(self::$exampleGroups as $name){
                    $key = sanitize_title($name);
                    if(isset($this->groups[$key])) continue;
                    $this->groups[$key] = $name;
                }
                $this->groups = $this->updateGroups($this->groups);
                break;
        }
        header('Location: ' . $_SERVER['HTTP_REFERER']);
    }
    
    public function addPagesBulkActions($actions, $groups){
        foreach($groups as $key => $group){
            $actions['flx_assign_page_to_group_'.$key] = 'Assign to '.$group;
        }
        return $actions;
    }
    
    public function pageGroupsFilter($groups){
        $selected_status = (!empty($_GET['post_status']) && ($_GET['post_status'] != 'all')) ? $_GET['post_status'] : false;
        $selected_group = (!empty($_GET['page_group'])) ? $_GET['page_group'] : 'all_groups';
        $group_count = $this->getGroupsCounts($selected_status);
            
        $post_status = ($selected_status) ? '&post_status='.$selected_status : false;

        $groups[] =
            '<a href="options-general.php?page=flx_dashboard_groups&tab=pages">⚙️</a> 
            <b>Page groups : </b> '
            .(($selected_group != 'all_groups')
                ? '<a href="edit.php?post_type=page'.$post_status.'&page_group=all_groups">'.__('All').'</a>'
                : '<b>'.__('All').'</b>');
        
        if(
            !isset(SettingsModule::$settings['display_unclassified'])
            || (SettingsModule::$settings['display_unclassified'] == 1)
        ){
            $num_posts = wp_count_posts('page', 'readable');
            if(!$selected_status){
                $total_posts = array_sum((array)$num_posts);
                foreach(get_post_stati(['show_in_admin_all_list' => false]) as $state) {
                    $total_posts -= $num_posts->$state;
                }
            }else{
                $total_posts = (isset($num_posts->$selected_status)) ? $num_posts->$selected_status : 0;
            }

            $total_assigned = array_sum(array_column($group_count, 'count')) ?: 0;
            $unclassified_nb = $total_posts - $total_assigned;

            $groups[] = (($selected_group != 'unclassified')
                ? '<a href="edit.php?post_type=page'.$post_status.'&page_group=unclassified">'.__('Unclassified').' ('.$unclassified_nb.')</a>'
                : '<b>'.__('Unclassified').' ('.$unclassified_nb.')</b>');
        }

        $hide_empty = (
            isset(SettingsModule::$settings['hide_empty_groups'])
            && (SettingsModule::$settings['hide_empty_groups'] == 1)
        );

        foreach(SettingsModule::$data['PAGE_GROUPS_OPTION'] as $key => $val){
            $count = (isset($group_count[$key])) ? $group_count[$key]->count : 0;
            if($hide_empty && ($count < 1)) continue;
            $count = ($count) ? ' ('.$count.')' : ' (0)';
            $groups[$key] = (urldecode($key) != $selected_group)
                ? '<a href="edit.php?post_type=page'.$post_status.'&page_group='.$key.'">'.$val.$count.'</a>'
                : '<b>'.$val.$count.'</b>';
        }

        return $groups;
    }
    
    public function pageFilter($query){
        if(!isset($_GET['post_type']) || ($_GET['post_type'] != 'page')) return $query;
        if(!$query->is_main_query()) return $query;
        global $pagenow;
        if(is_admin() && ($pagenow === 'edit.php')){
            $page_group = (isset($_GET['page_group'])) ? sanitize_title($_GET['page_group']) : false;
            if(!$page_group || ($page_group == 'all_groups')) return $query;

            if($page_group === 'unclassified'){
                $query->set('meta_query', [
                    [
                        'key' => SettingsModule::PAGE_GROUP_META,
                        'value' => '',
                        'compare' => 'NOT EXISTS',
                    ]
                ]);
            }else{
                $query->set('meta_query', [
                    [
                        'key' => SettingsModule::PAGE_GROUP_META,
                        'value' => $page_group,
                        'compare' => 'IN'
                    ]
                ]);
            }
        }
        return $query;
    }
    
   public function addPageGroupColumns($columns) {
       return array_slice($columns, 0, 2)
            + ['PageGroup' => 'Group']
            + array_slice($columns, 2);
   }
   public function addPageGroupCell($colname, $key) {
        if($colname != 'PageGroup') return;
        if(!$group = $this->getPageGroup($key)){
            echo '<em>'.__('None').'</em>';
            return;
        }
        echo ($this->getGroupName($group->page_group)) ?: '<em>'.__('None').'</em>';
   }

    public function onAssignGroup($redirect_to, $doaction, $post_ids) {
        if(substr($doaction, 0, 24) !== 'flx_assign_page_to_group') return $redirect_to;
        $group = sanitize_title(str_replace('flx_assign_page_to_group_', '', $doaction));
        foreach($post_ids as $post_id){
            update_post_meta($post_id, SettingsModule::PAGE_GROUP_META, $group);
        }
        return $redirect_to;
    }

    public function getPageGroup($key){
        return (
            isset(
                SettingsModule::$data['PAGE_GROUP_META'][$key]
            )
        ) ? SettingsModule::$data['PAGE_GROUP_META'][$key] : false;
    }

    public function getGroupName($key){
        return (
            isset(
                SettingsModule::$data['PAGE_GROUPS_OPTION'][$key]
            )
        ) ? SettingsModule::$data['PAGE_GROUPS_OPTION'][$key] : false;
    }
    
    private function onRemoveGroup($groups){
        $key = sanitize_title($_POST['flxpagegroups_action']['remove_page_group']);
        if(!isset($groups[$key])) return $groups;
        unset($groups[$key]);
        update_option(SettingsModule::PAGE_GROUPS_OPTION, $groups, false);
        return $groups;
    }
    
    private function onCreateGroup($groups){
        $group_name = (!empty($_POST['flxpagegroups_action']['add_page_group']))
            ? $_POST['flxpagegroups_action']['add_page_group']
            : false;
        if(!$group_name) return $groups;
        $groups[sanitize_title($group_name)] = $group_name;
        return $this->updateGroups($groups);
    }

    private function updateGroups($groups = []){
        update_option(SettingsModule::PAGE_GROUPS_OPTION, $groups, false);
        return $groups;
    }

    private function setGroupsTable($groups = []){
        $table = '';
        if(empty($groups)){
            $table .=
                '<tr> 
                    <td>No page groups</td>
                </tr>';
        }
        $groups_count = $this->getGroupsCounts();
        foreach($groups as $key => $group){
            $count = (isset($groups_count[$key])) ? $groups_count[$key]->count : '0';
            $table .=
                '<tr> 
                <th>'.htmlspecialchars($group).' ('.$count.')</th>
                    <td class="flx-group-action">
                        <button type="submit" name="flxpagegroups_action[remove_page_group]" value="'.htmlspecialchars($group).'" title="Delete this group">❌</button>
                    </td>
                </tr>';
        }
        $table =
            '<h2>Pages Groups</h2>
            <form method="POST">
                <table class="wp-list-table widefat striped flx_groups_list">
                  '.$table.'
                </table>
            </form>';
        $this->groupsTable = $table;
    }
    
    private function setAddForm(){
        $this->addForm =
            '<form method="POST">
                <table class="wp-list-table widefat striped flx_add_group_form">
                    <tr>
                        <th>
                            <b>Add a page group :</b>
                        </th>
                        <td>
                            <input type="text" name="flxpagegroups_action[add_page_group]" id="flxdg-groupname_input" placeholder="Group name"/>
                        </td>
                        <td class="add_group_btn">
                            <button class="button button-primary" type="submit">Add</button>
                        </td>
                    </tr>
                </table>
            </form>';
    }
    
    public function display(){
        echo $this->groupsTable,
            '<br>',$this->addForm,
            '<br><br>
            <table class="wp-list-table widefat striped">
                <tr>
                    <th>Insert example groups <em>(skip existing) : </em></th>
                    <td>'.implode(', ', self::$exampleGroups).'</td>
                    <td>
                        <form method="POST">
                            <button type="submit" name="flxpagegroups_action[flxdg-insert-default-page-groups]" value="1">Insert</button>
                        </form>
                    </td>
                </tr>
            </table>';
    }
}