<?php
namespace FLXDashboardGroups;

class PluginsGroupsModule{
    
    public $groupsTable;
    public $addForm;
    public $assignationsForm;
    
    public $plugins = [];
    public $groups = [];
    public $assignations = [];

    public static $exampleGroups = [
        '🛡 Security',
        '🎨 Design',
        '🛒 Shop',
        '🔥 Dev',
        '🏆 Nice plugin'
    ];

    function __construct($plugins = []){
        $this->groups = $groups = SettingsModule::$data['PLUGIN_GROUPS_OPTION'];
        $this->assignations = SettingsModule::$data['PLUGIN_ASSIGNED_GROUPS_OPTION'];
        
        foreach($plugins as $key => $plugin){
            $plugin_name = sanitize_title($plugin['Name']);
            if(empty($this->assignations[$plugin_name])) continue;
            $assigned = $this->assignations[$plugin_name];
            $plugins[$key]['PluginGroup'] = $this->groups[$assigned] ?: null;
        }
        $this->plugins = $plugins;

        if(!empty($_POST['flx_groups_action'])) $this->onAction();
        
        $this->setGroupsTable($groups, $this->assignations);
        $this->setAddForm();
        $this->setAssignationsForm();
        
        add_action('all_plugins', [$this, 'pluginFilter'], 20);
        add_action('views_plugins', [$this, 'pluginGroupsFilter']);
        
        add_filter( 'bulk_actions-plugins',  [$this, 'addPluginsBulkActions']);
        add_filter('manage_plugins_columns', [$this, 'addPluginGroupColumns']);
        
        add_filter('manage_plugins_custom_column', [$this, 'addGroupColumn'], 10, 2);
        add_filter('handle_bulk_actions-plugins', [$this, 'onBulkAction'], 10, 3);
        
        add_filter(
            (is_network_admin() ? 'network_admin_' : '')
            .'plugin_action_links_flx-dashboard-groups/flx_dashboard_groups.php',
            [$this, 'addSettingsLink']
        );
        
    }
    
    private function onAction(){
        $action = array_keys($_POST['flx_groups_action'])[0];
        switch($action){
            default: break;
            
            case 'remove_plugin_group':
                $this->groups = $this->onRemoveGroup($this->groups);
                break;
            
            case 'add_plugin_group':
                $this->groups = $this->onCreateGroup($this->groups);
                break;
            
            case 'save_plugins_groups':
                if(empty($_POST['assigned_plugins'])) break;
                $this->assignations = array_filter($_POST['assigned_plugins']);
                update_option(SettingsModule::PLUGIN_ASSIGNED_GROUPS_OPTION, $this->assignations, false);
                break;
            
            case 'flxdg-insert-default-plugin-groups':
                foreach(self::$exampleGroups as $name){
                    $key = sanitize_title($name);
                    if(isset($this->groups[$key])) continue;
                    $this->groups[$key] = $name;
                }
                $this->groups = $this->updateGroups($this->groups);
                break;
        }
        header('Location: ' . $_SERVER['HTTP_REFERER']);
    }
    
    public function addPluginsBulkActions($actions){
        foreach($this->groups as $key => $group){
            $actions['flx_assign_plugin_to_group_'.$key] = 'Assign to '.$group;
        }
        $actions['flx_unset_plugin_group'] = 'Unassign group';
        return $actions;
    }

    public function addPluginGroupColumns($columns) {
        $desc = $columns["description"];
        unset($columns["description"]);
        $columns["PluginGroup"] = "Group";
        $columns["description"] = $desc;
        return $columns;
    }

    public function addGroupColumn($colname, $plugin_key) {
        if($colname != 'PluginGroup') return;
        echo (empty($this->plugins[$plugin_key]['PluginGroup']))
            ? '<em>'.__('None').'</em>'
            : htmlspecialchars(urldecode($this->plugins[$plugin_key]['PluginGroup']));
   }

    public function addSettingsLink($links){
        $links[] = '<a href="' .admin_url( 'options-general.php?page=flx_dashboard_groups' ) .'">' . __('Settings') . '</a>';
        return $links;
    }
    
    public function pluginGroupsFilter($links){
        $selected_status = (!empty($_GET['plugin_status'])) ? $_GET['plugin_status'] : 'all';
        $selected_group = (!empty($_GET['plugin_group'])) ? $_GET['plugin_group'] : 'all_groups';
        $links[] = ' <a href="options-general.php?page=flx_dashboard_groups">⚙️</a> 
        <b>Plugins groups : </b> '
        .(($selected_group != 'all_groups')
            ? '<a href="plugins.php?plugin_status='.$selected_status.'&plugin_group=all_groups">'.__('All').'</a>'
            : '<b>'.__('All').'</b>');
        
        $assignations = $this->assignations;
        $plugins =$this->plugins;
        global $s;
        if(strlen($s) > 0) $selected_status = 'search';
        if($selected_status && ($selected_status !== 'all')){
            switch($selected_status){
                default: break;
                case 'search':
                    $plugins = array_filter($plugins, function ($plugin) use($s, &$assignations) {
                        foreach($plugin as $value){
                            if(
                                is_string($value)
                                && (stripos(strip_tags($value), urldecode($s)) !== false)
                            ) return true;
                        }
                        unset($assignations[sanitize_title($plugin['Name'])]);
                        return false;
                    });
                    break;
                case 'active':
                    foreach($plugins as $key => $plugin){
                        if(is_plugin_active($key)) continue;
                        unset($plugins[$key]);
                        unset($assignations[sanitize_title($plugin['Name'])]);
                    }
                    break;
                case 'inactive':
                    foreach($plugins as $key => $plugin){
                        if(!is_plugin_active($key)) continue;
                        unset($plugins[$key]);
                        unset($assignations[sanitize_title($plugin['Name'])]);
                    }
                    break;
                case 'recently_activated':
                    $recently_activated = get_option('recently_activated', []);
                    foreach($plugins as $key => $plugin){
                        if(isset($recently_activated[$key]) && !is_plugin_active($key)) continue;
                        unset($plugins[$key]);
                        unset($assignations[sanitize_title($plugin['Name'])]);
                    }
                    break;
                case 'upgrade':
                case 'mustuse':
                case 'dropins':
                    $plugins = [];
                    $assignations = [];
                    break;
            }
        }

        $groups_plugins = $this->getGroupsPlugins($assignations);
        if(
            !isset(SettingsModule::$settings['display_unclassified'])
            || (SettingsModule::$settings['display_unclassified'] == 1)
        ){
            $total_nb = count($plugins);
            $total_assigned = count($groups_plugins);
            $unclassified_nb = $total_nb - $total_assigned;
            
            $links[] = (($selected_group != 'unclassified')
                ? (($unclassified_nb > 0)
                    ? '<a href="plugins.php?plugin_status='.$selected_status.'&plugin_group=unclassified">'.__('Unclassified').' ('.$unclassified_nb.')</a>'
                    : '<span>'.__('Unclassified').' (0)</b></span>')
                : '<b>'.__('Unclassified').' ('.$unclassified_nb.')</b>');
        }
        
        foreach($this->groups as $key => $val){
            $count = (isset($groups_plugins[$key])) ? count($groups_plugins[$key]) : 0;
            $links[$key] = (urldecode($key) != $selected_group)
                ? (($count > 0)
                    ? '<a href="plugins.php?plugin_status='.$selected_status.'&plugin_group='.$key.'">'.$val.' ('.$count.')</a>'
                    : '<span>'.$val.' (0)</span>')
                : '<b>'.$val.' ('.$count.')</b>';
        }
        return $links;
    }
    
    public function pluginFilter($plugins){
        $selected_status = (!empty($_GET['plugin_status']) && ($_GET['plugin_status']!== 'all'))
            ? $_GET['plugin_status'] : false;
        
        if(!isset($_GET['plugin_group'])) return $plugins;
        
        $current_group = sanitize_title($_GET['plugin_group']);
        if($current_group == 'all_groups') return $plugins;

        $assignations = $this->assignations;

        if($current_group === 'unclassified'){
            foreach($plugins as $key => $plugin){
                $name = sanitize_title($plugin['Name']);
                if(isset($assignations[$name])){
                    unset($plugins[$key]);
                    unset($assignations[$name]);
                }
            }
            return $plugins;
        } else {
            $assignations = $this->getGroupsPlugins($this->assignations);
            if (!isset($assignations[$current_group])) return [];

            $group_plugins = $assignations[$current_group];
            foreach ($plugins as $key => $plugin) {
                $name = sanitize_title($plugin['Name']);
                if (!in_array($name, $group_plugins)) {
                    unset($plugins[$key]);
                    unset($assignations[$name]);
                }
            }
            return $plugins;
        }
    }


    public function onBulkAction($redirect_to, $doaction, $selected_plugins) {
        $redirect_to = add_query_arg(false, false);
        switch ($doaction) {
            case 'flx_unset_plugin_group':
                foreach ($selected_plugins as $plugin_key) {
                    $plugin_name = sanitize_title($this->plugins[$plugin_key]['Name']);
                    if (!isset($this->assignations[$plugin_name])) continue;
                    unset($this->assignations[$plugin_name]);
                }
                update_option(SettingsModule::PLUGIN_ASSIGNED_GROUPS_OPTION, $this->assignations, false);
                break;

            case (substr($doaction, 0, 26) === 'flx_assign_plugin_to_group'):
                $group = str_replace('flx_assign_plugin_to_group_', '', $doaction);
                foreach ($selected_plugins as $plugin_key) {
                    if (!isset($this->plugins[$plugin_key])) continue;
                    $plugin_name = sanitize_title($this->plugins[$plugin_key]['Name']);
                    $this->assignations[$plugin_name] = $group;
                }
                update_option(SettingsModule::PLUGIN_ASSIGNED_GROUPS_OPTION, $this->assignations, false);
                break;

            default:
                return $redirect_to;
        }
        return $redirect_to;
    }


    public function getGroupsPlugins($assignations = []){
        $group_plugins = [];
        foreach ($assignations as $plugin => $assigned_group) {
            if (!isset($group_plugins[$assigned_group])) $group_plugins[$assigned_group] = [];
            $group_plugins[$assigned_group][] = $plugin;
        }
        return $group_plugins;
    }


    private function onRemoveGroup($groups){
        $key = sanitize_title($_POST['flx_groups_action']['remove_plugin_group']);
        if (!isset($groups[$key])) return $groups;
        unset($groups[$key]);
        update_option(SettingsModule::PLUGIN_GROUPS_OPTION, $groups, false);
        return $groups;
    }


    private function onCreateGroup($groups){
        $group_name = (!empty($_POST['flx_groups_action']['add_plugin_group']))
            ? $_POST['flx_groups_action']['add_plugin_group']
            : false;
        if(!$group_name) return $groups;
        $groups[sanitize_title($group_name)] = $group_name;
        return $this->updateGroups($groups);
    }


    private function updateGroups($groups = []){
        update_option(SettingsModule::PLUGIN_GROUPS_OPTION, $groups, false);
        return $groups;
    }


    public function getGroupSelectOptions($groups, $selected = null){
        $groups_list = '<option value="">Aucun</option>';
        foreach ($groups as $key => $group) {
            $groups_list .= '<option value="' . $key . '"' . (($key == $selected) ? ' selected' : '') . '>' . $group . '</option>';
        }
        return $groups_list;
    }

    private function setGroupsTable(){
        $table = '';
        $groups_plugins = $this->getGroupsPlugins($this->assignations);
        foreach ($this->groups as $key => $group) {
            $group_plugins = (isset($groups_plugins[$key])) ? $groups_plugins[$key] : [__('None')];
            $group_plugins = '(' . implode(', ', $group_plugins) . ')';
            $table .=
                '<tr class="inline-edit-row"> 
                    <th>' . htmlspecialchars($group) . '</th>
                    <td>' . $group_plugins . '</td>
                    <td class="flx-group-action">
                        <button type="submit" name="flx_groups_action[remove_plugin_group]" value="' . htmlspecialchars($group) . '" title="Delete this group">❌</button>
                    </td>
                </tr>';
        }
        $table =
            '<h2>Plugins Groups</h2>
            <form method="POST">
                <table class="wp-list-table widefat striped flx_groups_list">
                    <thead>
                        <th>Name</th>
                        <th>Assigned plugins</th>
                        <th>Delete</th>
                    </thead>
                  ' . $table . '
                </table>
            </form>';
        $this->groupsTable = $table;
    }

    private function setAddForm(){
        $this->addForm =
            '<form method="POST">
                <table class="wp-list-table widefat striped flx_add_group_form">
                    <tr>
                        <th colspan=2>
                            <b>Add a plugin group :</b>
                        </th>
                        <td>
                            <input type="text" name="flx_groups_action[add_plugin_group]" class="regular-text ltr" placeholder="Group name"/>
                        </td>
                        <td class="add_group_btn">
                            <button class="button button-primary" type="submit">Add</button>
                        </td>
                    </tr>
                </table>
            </form>';
    }


    private function setAssignationsForm(){
        $form = '';
        foreach ($this->plugins as $plugin) {
            $key = sanitize_title($plugin['Name']);
            $selected = (isset($this->assignations[$key])) ? $this->assignations[$key] : null;
            $form .=
                '<tr>
                    <th>' . $plugin['Name'] . ' - ' . $key . '</th>
                    <td>
                    <select name="assigned_plugins[' . $key . ']">'
                        . $this->getGroupSelectOptions($this->groups, $selected) .
                    '</select>
                    </td>
                </tr>';
        }
        $form =
        '<form method="POST">
            <table class="wp-list-table widefat striped">
                <thead>
                    <tr>
                        <th class="column-primary">Plugin</th>
                        <th>Group</th>
                    </tr>
                </thead>
                ' . $form . '
                ' . ((empty($form))
                    ? '<tr>
                        <td>No plugins</td>
                    </tr>'
                    : '<tr>
                        <td></td>
                        <td>
                            <button name="flx_groups_action[save_plugins_groups]" class="button button-primary" type="submit">Save</button>
                        </td>
                </tr>'
                ).'
            </table>
        </form>';
        $this->assignationsForm = $form;
    }


    public function display() {
        echo $this->groupsTable,
            '<br>',$this->addForm,
            '<br>',$this->assignationsForm,
            '<br><br>
            <table class="wp-list-table widefat striped">
                <tr>
                    <th>Insert example groups <em>(skip existing) : </em></th>
                    <td>' . implode(', ', self::$exampleGroups) . '</td>
                    <td>
                        <form method="POST">
                            <button type="submit" name="flx_groups_action[flxdg-insert-default-plugin-groups]" value="1">Insert</button>
                        </form>
                    </td>
                </tr>
            </table>';
    }
}
